function [newmu,newse] = mcmc(efile,sefile,replicates,num_chain,num_sample,display)
%
% mcmc  Combine replicates for each condition
% 
% INPUT:
%
%  efile      Character string, the full csv file name of gene expression 
%             data file.
%  sefile     Character string, the full csv file name of the file containing 
%             standard deviation of gene expression.
%             The two data files are comma-separated and contain a tile, 
%             column names and a n x c matrix of data. n is the number of 
%             probe-sets and c is the number of chips.
%  replicates Vector, indicating replicates for each condition, for example, 
%             [1 1 1 2 2 2] means the first 3 chips belong to condition 1 and
%             the last 3 chips belong to condition 2.
%  num_chain  Integer, the number of simulation chains, default 10.
%  num_sample Integer, the basic number of simulations for each chain, default
%             200.
%  display    Boolean, value of 1 display the process of simulations gene
%             by gene.
%
% OUTPUT:
%
%  newmu  Matrix, the combined new expression values for each condition.
%  newse  Matrix, the standard deviation of the combined expression values.
%

global expr stdev rep muj accp

if nargin < 3
    fprintf('Input the replicate vector REPLICATES')
    return
end
rep = replicates;

e = csvread(efile,1,1);
se = csvread(sefile,1,1);

se = se.^2;

if nargin<4
    num_chain = 10;
    num_sample = 200;
    display = 0;
elseif nargin < 5
    num_sample = 200;
    display = 0;
end

cond = max(rep);
chip = length(rep);

sig0 = zeros(1,num_chain);
newmu = zeros(length(e),cond);
newse = zeros(length(e),cond);
lambda_m = zeros(length(e),1);
accpall = zeros(length(e),1);
cond = max(rep);
for i=1:1:length(e)
    expr = e(i,:);
    stdev = se(i,:);
    
    accp = 0;
    
    par_samp = cell(1,cond+4);
    for j=1:1:cond+3
        par_samp{j} = zeros(num_sample,num_chain);
    end
    
    muj = zeros(1,cond);
    
    % initial values
    for k=1:1:num_chain
        if k==1
            n0=0;
        else
            n0=1;
        end
        x = zeros(1,cond);
        for j=1:1:cond
            ind = find(rep==j);
            muj(j) = mean(expr(ind));
            % muj
            par_samp{j}(1,k) = muj(j)+n0*normrnd(0,1)*muj(j);
            v(j) = var(expr(ind));
        end
        % mu0
        par_samp{cond+1}(1,k) = mean(muj)*(1+n0*normrnd(0,1));
        % eta0
        par_samp{cond+2}(1,k) = abs((1+n0*normrnd(0,1))*gamrnd(cond/2+0.001,1/(sum((muj-mean(muj)).^2)/2+0.001)));
        % lambda0
        [ll logf sig0(k)] = updatescale(-log(mean(v)),0);
        par_samp{cond+3}(1,k) = abs(ll*(1+n0*normrnd(0,1)));
        par_samp{cond+4}(1,k) = logf;
    end

    % simulations
    R = 10*ones(1,num_chain);
    n = 1;
    while(max(R)>1.1)
        if n == 1
            s0 = 2;
        else
            s0 = 1;
        end
        for s=s0:1:num_sample
            n = n+1;
            
            muj = zeros(1,cond);
            mu = par_samp{cond+1}(n-1,:);
            eta = par_samp{cond+2}(n-1,:);
            lambda = par_samp{cond+3}(n-1,:);
            for k=1:1:num_chain
                for j=1:1:cond
                    ind = find(rep==j);
                    sig_samp = sqrt(1/(eta(k)+sum(1./(1/lambda(k)+stdev(ind)))));
                    mu_samp = (eta(k)*mu(k)+sum(expr(ind)./(1/lambda(k)+stdev(ind))))*sig_samp^2;
                    par_samp{j}(n,k) = normrnd(mu_samp, sig_samp);
                    muj(j) = par_samp{j}(n,k);
                end
                par_samp{cond+1}(n,k) = normrnd(mean(muj),sqrt(1/(cond*eta(k))));
                mu(k) = par_samp{cond+1}(n,k);
                eta(k) = gamrnd(cond/2+0.001,1/(sum((muj-mu(k)).^2)/2+0.001));
                par_samp{cond+2}(n,k) = eta(k);

                num_all_sample = num_chain*n/2;
                accpall(i) = accp/(num_all_sample*2);

                if n<num_sample/2 && abs(accpall(i)-0.44)>0.02 
                    [ll logf sig0(k)] = updatescale(-log(lambda(k)),1);
                else
                    [ll logf] = samplelambda(-log(lambda(k)),sig0(k));
                end
                par_samp{cond+3}(n,k) = ll;
                par_samp{cond+4}(n,k) = logf;
            end
        end
        R = calculateR(par_samp);
    end
    
    num_all_sample = num_chain*n/2;
    accpall(i) = accp/(num_all_sample*2);
    
    for k=1:1:cond
        temp = reshape(par_samp{k}((n/2+1):n,:),1,num_all_sample);
        newmu(i,k) = mean(temp);
        newse(i,k) = std(temp);
    end
    lambda_m(i) = sqrt(1/mean(reshape(par_samp{cond+3}((n/2+1):n,:),1,num_all_sample)));
    
    if display==1
        fprintf('gene: %d simulations: %d acceptance rate: %f\n',i,n,accpall(i))
    end
end

